﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace GliderRemoteCompat {
	public class Win32Input {
		[DllImport("user32.dll", EntryPoint = "SendInput", SetLastError = true)]
		static extern uint SendInput(uint nInputs, INPUT[] pInputs, int cbSize);

		[DllImport("user32.dll", EntryPoint = "GetMessageExtraInfo", SetLastError = true)]
		static extern IntPtr GetMessageExtraInfo();

		private enum InputType {
			INPUT_MOUSE = 0,
			INPUT_KEYBOARD = 1,
			INPUT_HARDWARE = 2,
		}

		[Flags()]
		private enum MOUSEEVENTF {
			MOVE = 0x0001,  // mouse move
			LEFTDOWN = 0x0002,  // left button down
			LEFTUP = 0x0004,  // left button up
			RIGHTDOWN = 0x0008,  // right button down
			RIGHTUP = 0x0010,  // right button up
			MIDDLEDOWN = 0x0020,  // middle button down
			MIDDLEUP = 0x0040,  // middle button up
			XDOWN = 0x0080,  // x button down
			XUP = 0x0100,  // x button down
			WHEEL = 0x0800,  // wheel button rolled
			VIRTUALDESK = 0x4000,  // map to entire virtual desktop
			ABSOLUTE = 0x8000,  // absolute move
		}

		[Flags()]
		private enum KEYEVENTF {
			EXTENDEDKEY = 0x0001,
			KEYUP = 0x0002,
			UNICODE = 0x0004,
			SCANCODE = 0x0008,
		}

		[StructLayout(LayoutKind.Sequential)]
		private struct MOUSEINPUT {
			public int dx;
			public int dy;
			public int mouseData;
			public int dwFlags;
			public int time;
			public IntPtr dwExtraInfo;
		}

		[StructLayout(LayoutKind.Sequential)]
		private struct KEYBDINPUT {
			public short wVk;
			public short wScan;
			public int dwFlags;
			public int time;
			public IntPtr dwExtraInfo;
		}

		[StructLayout(LayoutKind.Sequential)]
		private struct HARDWAREINPUT {
			public int uMsg;
			public short wParamL;
			public short wParamH;
		}

		[StructLayout(LayoutKind.Explicit)]
		private struct INPUT {
			[FieldOffset(0)]
			public int type;
			[FieldOffset(4)]
			public MOUSEINPUT mi;
			[FieldOffset(4)]
			public KEYBDINPUT ki;
			[FieldOffset(4)]
			public HARDWAREINPUT hi;
		}
		/// <summary>
		/// This function moves the cursor to a specific point at the screen.
		/// </summary>
		/// <param name="x">X coordinate of the posotion as pixel</param>
		/// <param name="y">Y coordinate of the posotion as pixel</param>
		/// <returns>Returns 0 if there was an error otherwise 1.</returns>
		public static bool Move(int x, int y) {
			// Bildschirm Auflösung
			float ScreenWidth = Screen.PrimaryScreen.Bounds.Width;
			float ScreenHeight = Screen.PrimaryScreen.Bounds.Height;

			INPUT input_move = new INPUT();
			input_move.mi.dx = (int)Math.Round(x * (65535 / ScreenWidth), 0);
			input_move.mi.dy = (int)Math.Round(y * (65535 / ScreenHeight), 0);
			input_move.mi.mouseData = 0;
			input_move.mi.dwFlags = (int)(MOUSEEVENTF.MOVE | MOUSEEVENTF.ABSOLUTE);

			INPUT[] input = { input_move };
			return input.Length == SendInput(1, input, Marshal.SizeOf(input_move));
		}

		public static bool LeftClick() {
			return Click(false);
		}

		public static bool RightClick() {
			return Click(true);
		}

		/// <summary>
		/// This function simulates a simple mouseclick at the current cursor position.
		/// </summary>
		/// <returns>All right if it is 2. All below indicates an error.</returns>
		public static bool Click(bool rightClick) {
			INPUT input_down = new INPUT();
			input_down.mi.dx = 0;
			input_down.mi.dy = 0;
			input_down.mi.mouseData = 0;
			input_down.mi.dwFlags = (int)(rightClick ? MOUSEEVENTF.RIGHTDOWN : MOUSEEVENTF.LEFTDOWN);

			INPUT input_up = input_down;
			input_up.mi.dwFlags = (int)(rightClick ? MOUSEEVENTF.RIGHTUP : MOUSEEVENTF.LEFTUP);

			INPUT[] input = { input_down, input_up };
			return input.Length == SendInput(2, input, Marshal.SizeOf(input_down));
		}
	}
}
